////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2014-2022, Lawrence Livermore National Security, LLC.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the LBANN Research Team (B. Van Essen, et al.) listed in
// the CONTRIBUTORS file. <lbann-dev@llnl.gov>
//
// LLNL-CODE-697807.
// All rights reserved.
//
// This file is part of LBANN: Livermore Big Artificial Neural Network
// Toolkit. For details, see http://software.llnl.gov/LBANN or
// https://github.com/LLNL/LBANN.
//
// Licensed under the Apache License, Version 2.0 (the "Licensee"); you
// may not use this file except in compliance with the License.  You may
// obtain a copy of the License at:
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
// implied. See the License for the specific language governing
// permissions and limitations under the license.
////////////////////////////////////////////////////////////////////////////////

#define LBANN_WEIGHTED_SUM_LAYER_INSTANTIATE
#include "lbann/layers/transform/weighted_sum.hpp"

#include "lbann/proto/proto_common.hpp"
#include "lbann/utils/protobuf.hpp"

#include <lbann.pb.h>

namespace lbann {

template <typename TensorDataType, data_layout Layout, El::Device Device>
std::unique_ptr<Layer>
build_weighted_sum_layer_from_pbuf(lbann_comm* comm,
                                   lbann_data::Layer const& proto_layer)
{
  using LayerType = weighted_sum_layer<TensorDataType, Layout, Device>;
  LBANN_ASSERT_MSG_HAS_FIELD(proto_layer, weighted_sum);
  const auto& params = proto_layer.weighted_sum();
  const auto& scaling_factors =
    protobuf::to_vector<DataType>(params.scaling_factors());
  return std::make_unique<LayerType>(comm, scaling_factors);
}

#define PROTO_DEVICE(T, Device) \
  template class weighted_sum_layer<T, data_layout::DATA_PARALLEL, Device>; \
  template class weighted_sum_layer<T, data_layout::MODEL_PARALLEL, Device>; \
  LBANN_LAYER_BUILDER_ETI(weighted_sum, T, Device)

#include "lbann/macros/instantiate_device.hpp"

}// namespace lbann
